#include "StdAfx.h"
#include "AesTool.h"
#include "Pbkdf2HmacSha1.h"



CAesTool::CAesTool(void):mMode(AES256)
{
}


CAesTool::~CAesTool(void)
{
}

int CAesTool::init(const unsigned char *key, int keylen)
{
	if (keylen != GetKeySize()) {
		return -1;
	}
	AesCtr2_Init(&mAes);
	Aes_SetKey_Enc(mAes.aes + mAes.offset + 8, key, keylen);
	return 0;
}

int CAesTool::encrypt(unsigned char *data, int len)
{
	AesCtr2_Code(&mAes, data, len);
	hmac.Update(data, len);
	return len;
}

int CAesTool::decrypt(unsigned char *data, int len)
{
	hmac.Update(data, len);
	AesCtr2_Code(&mAes, data, len);
	return len;
}


void CAesTool::BytesToBeUInt32s(const unsigned char *src, unsigned long *dest, unsigned destSize)
{
	for (unsigned i = 0; i < destSize; i++)
		dest[i] =
		((unsigned long)(src[i * 4 + 0]) << 24) |
		((unsigned long)(src[i * 4 + 1]) << 16) |
		((unsigned long)(src[i * 4 + 2]) <<  8) |
		((unsigned long)(src[i * 4 + 3]));
}

void CAesTool::pbkdf2Hmac(const unsigned char *password, int passwordlen, 
	const unsigned char *salt, int saltlen, unsigned char *out, int outlen)
{
	unsigned long keysTotalSize = (unsigned long)outlen;
	unsigned long buf32[(2 * MAX_KEY_SIZE + KEY_PASSWORD_VERIFY_SIZE + 3) / 4];
	unsigned long key32SizeTotal = (keysTotalSize + 3) / 4;
	unsigned long salt32[kSaltSizeMax * 4];
	unsigned long saltSizeInWords = saltlen / 4;
	BytesToBeUInt32s(salt, salt32, saltSizeInWords);
	Pbkdf2Hmac32(password, passwordlen,
		salt32, saltSizeInWords, NUMITERATIONS,buf32, key32SizeTotal);
	for (int j = 0; j < outlen; j++) {
		out[j] = (unsigned char)(buf32[j / 4] >> (24 - 8 * (j & 3)));
	}
}

int CAesTool::HmacInit(const unsigned char *key, int keylen)
{
	hmac.SetKey(key, keylen);
	return 0;
}

int CAesTool::update(const unsigned char *data, int len)
{
    hmac.Update(data, len);
    return len;
}

int CAesTool::doFinal(unsigned char *out, int len)
{
	hmac.Final(out, len);
	return len;
}
