// Crypto/WzAes.cpp
/*
This code implements Brian Gladman's scheme
specified in password Based File Encryption Utility.

Note: you must include MyAes.cpp to project to initialize AES tables
*/

#include "StdAfx.h"

//#include "../Common/StreamObjects.h"
//#include "../Common/StreamUtils.h"

//#include "Pbkdf2HmacSha1.h"
//#include "RandGen.h"
#include "WzAes.h"
#include "Pbkdf2HmacSha1.h"

// define it if you don't want to use speed-optimized version of Pbkdf2HmacSha1
// #define _NO_WZAES_OPTIMIZATIONS


const unsigned kAesKeySizeMax = 32;

CKeyInfo _key;
CAesCtr2 _aes;

static const UInt32 kNumKeyGenIterations = 1000;

void SetPassword(CKeyInfo &_key, const Byte *data, UInt32 size)
{
	_key.Passwordlen = size;
	memcpy(_key.Password, data, size);
}

void Init()
{
  AesCtr2_Init(&_aes);
  Aes_SetKey_Enc(_aes.aes + _aes.offset + 8, NULL, 0);
}

CAesCtr2::CAesCtr2()
{
  offset = ((0 - (unsigned)(ptrdiff_t)aes) & 0xF) / sizeof(UInt32);
}

void AesCtr2_Init(CAesCtr2 *p)
{
  UInt32 *ctr = p->aes + p->offset + 4;
  unsigned i;
  for (i = 0; i < 4; i++)
    ctr[i] = 0;
  p->pos = AES_BLOCK_SIZE;
}

void AesCtr2_Code(CAesCtr2 *p, Byte *data, SizeT size)
{
  unsigned pos = p->pos;
  UInt32 *buf32 = p->aes + p->offset;
  if (size == 0)
    return;
  if (pos != AES_BLOCK_SIZE)
  {
    const Byte *buf = (const Byte *)buf32;
    do {
      *data++ ^= buf[pos++];
	} while (--size != 0 && pos != AES_BLOCK_SIZE);
  }
  if (size >= 16)
  {
    SizeT size2 = size >> 4;
    g_AesCtr_Code(buf32 + 4, data, size2);
    size2 <<= 4;
    data += size2;
    size -= size2;
    pos = AES_BLOCK_SIZE;
  }
  if (size != 0)
  {
    unsigned j;
    const Byte *buf;
    for (j = 0; j < 4; j++)
      buf32[j] = 0;
    g_AesCtr_Code(buf32 + 4, (Byte *)buf32, 1);
    buf = (const Byte *)buf32;
    pos = 0;
    do
      *data++ ^= buf[pos++];
    while (--size != 0 && pos != AES_BLOCK_SIZE);
  }
  p->pos = pos;
}

UInt32 encrypt(CAesCtr2 &aes, Byte *data, UInt32 size)
{
	AesCtr2_Code(&aes, data, size);
	return size;
}

UInt32 decrypt(CAesCtr2 &aes, Byte *data, UInt32 size)
{
	AesCtr2_Code(&aes, data, size);
	return size;
}

int aes_encrypt(char *key, char *data, int len)
{
	CAesCtr2 aes;
	AesCtr2_Init(&aes);
	Aes_SetKey_Enc(aes.aes + aes.offset + 8, (const Byte*)key, 32);
	AesCtr2_Code(&aes, (Byte*)data, (UInt32)len);
	return len;
}

static void BytesToBeUInt32s(const Byte *src, UInt32 *dest, unsigned destSize)
{
	for (unsigned i = 0; i < destSize; i++)
		dest[i] =
		((UInt32)(src[i * 4 + 0]) << 24) |
		((UInt32)(src[i * 4 + 1]) << 16) |
		((UInt32)(src[i * 4 + 2]) <<  8) |
		((UInt32)(src[i * 4 + 3]));
}

void pbkdf2Hmac_aes(char *password, int psdlen, char *salt, int saltlen, char *out, int outlen)
{
	UInt32 keysTotalSize = 2 * 32 + kPwdVerifCodeSize;
	UInt32 buf32[(2 * kAesKeySizeMax + kPwdVerifCodeSize + 3) / 4];
	UInt32 key32SizeTotal = (keysTotalSize + 3) / 4;
	UInt32 salt32[kSaltSizeMax * 4];
	UInt32 saltSizeInWords = saltlen / 4;
	BytesToBeUInt32s((Byte*)salt, salt32, saltSizeInWords);
	Pbkdf2Hmac32(
		(Byte*)password, psdlen,
		salt32, saltSizeInWords,
		kNumKeyGenIterations,
		buf32, key32SizeTotal);
	for (UInt32 j = 0; j < keysTotalSize; j++)
		out[j] = (Byte)(buf32[j / 4] >> (24 - 8 * (j & 3)));
}
